-- The serial 4-bit adder solved as a dedicated processor

LIBRARY ieee ;
USE ieee.std_logic_1164.all ;

ENTITY Adder_4bit_serial IS  

PORT (
		CLK 		: 	IN 	STD_LOGIC;
		CD			: 	IN 	STD_LOGIC;
		A			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		B			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cin			: 	IN 	STD_LOGIC;		
		ST			: 	IN 	STD_LOGIC;
		S			: 	OUT	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cout		:  	OUT	STD_LOGIC;
		EOP			:  	OUT	STD_LOGIC
		);
END Adder_4bit_serial; 


ARCHITECTURE structural OF Adder_4bit_serial IS
    

COMPONENT  Datapath IS  	
	PORT (
		CLK			:	IN STD_LOGIC;
		CD			:	IN STD_LOGIC;      
		A			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		B			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cin			: 	IN 	STD_LOGIC;	

		Sel_Carry	: 	IN 	STD_LOGIC;	
		LDA			: 	IN 	STD_LOGIC_VECTOR (1 DOWNTO 0);		-- The same as the LDB
		LDW			: 	IN 	STD_LOGIC;							-- The same as the LDC
		LDS			: 	IN 	STD_LOGIC;							
		CE			:  	IN	STD_LOGIC;		
		TC4			:	OUT STD_LOGIC;		

		S			: 	OUT	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cout		:  	OUT	STD_LOGIC
		);
	END COMPONENT;
	
COMPONENT Control_Unit IS
	PORT(
			CLK, CD 	:	IN  STD_LOGIC;	
			TC4			:	IN  STD_LOGIC;
			ST			:	IN  STD_LOGIC;
		    LDA			:   OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			LDW			:	OUT std_logic;
			LDS			:	OUT std_logic;
			Sel_Carry	:	OUT std_logic;
			EOP			:	OUT std_logic;
			CE  		:	OUT std_logic
         );
	END COMPONENT;

   SIGNAL  LDA			: STD_LOGIC_VECTOR (1 DOWNTO 0);
   SIGNAL  Sel_Carry	: STD_LOGIC;     
   SIGNAL  TC4,CE		: STD_LOGIC;     
   SIGNAL  LDS, LDW		: STD_LOGIC;     
   
BEGIN
	
Chip1	:	Control_Unit
   	    PORT MAP (
	   		CLK			=> CLK,
			CD      	=> CD,
			ST      	=> ST,
			CE      	=> CE,
            TC4     	=> TC4,
            LDW			=> LDW,
            LDA     	=> LDA,
            LDS     	=> LDS,                 
			EOP     	=> EOP,
            Sel_Carry	=> Sel_Carry            
		);
		   
Chip2	: 	Datapath
  	    PORT MAP (
			CLK       	=> CLK,
			CD       	=> CD,
			LDA        	=> LDA,
			CE          => CE,
			LDS         => LDS,
			LDW			=> LDW,
            Sel_Carry	=> Sel_Carry,            
			A           => A,	
			B           => B,
			Cin			=> Cin,
			S           => S,
			Cout		=> Cout,
			TC4         => TC4
		);
		   
               
END structural ;