-- 4-bit unsignel multiplier datapath 
-- Structural and hiearchical design 

LIBRARY ieee;	
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Datapath IS  
	PORT (
		CLK			:	IN STD_LOGIC;
		CD			:	IN STD_LOGIC;      
		A			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		B			: 	IN 	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cin			: 	IN 	STD_LOGIC;	

		Sel_Carry	: 	IN 	STD_LOGIC;	
		LDA			: 	IN 	STD_LOGIC_VECTOR (1 DOWNTO 0);		-- The same as the LDB
		LDW			: 	IN 	STD_LOGIC;							-- The same as the LDC
		LDS			: 	IN 	STD_LOGIC;							
		CE			:  	IN	STD_LOGIC;		
		TC4			:	OUT STD_LOGIC;		

		S			: 	OUT	STD_LOGIC_VECTOR (3 DOWNTO 0);		
		Cout		:  	OUT	STD_LOGIC
		);
END Datapath;

ARCHITECTURE Structural OF Datapath IS
-- Components

	COMPONENT Shift_Data_Reg_4bits IS  
		PORT(	
			CD,CLK		: 	IN		std_logic;
			X			: 	IN 		std_logic_vector(3 downto 0);
			Y			: 	OUT 	std_logic_vector(3 downto 0);
			S			: 	IN	 	std_logic_vector(1 downto 0);
			RSI,LSI		: 	IN 		std_logic
		);
	END COMPONENT;

	COMPONENT MUX4 IS
		port(	
			E 	:	in std_logic;
			Ch0	:	in std_logic;
			Ch1	:	in std_logic;
			Ch2	:	in std_logic;
			Ch3	:	in std_logic;
			s	:	in std_logic_vector(1 downto 0);
			y	:	out std_logic
		);
	END COMPONENT;

	COMPONENT ONEBIT_ADDER IS  
		PORT (
			Ai,Bi, Ci	: IN STD_LOGIC;				
			So, Co		: OUT STD_LOGIC
		  );
	END COMPONENT;

	COMPONENT Freq_Div_4 IS
      Port (
		CLK 	: IN	STD_LOGIC;
		CD	 	: IN	STD_LOGIC;
		CE 		: IN	STD_LOGIC;
		TC4		: OUT	STD_LOGIC
	);         
	END COMPONENT;
	
	
-- Internal wires 
	SIGNAL QA, QB	:	STD_LOGIC_VECTOR(3 downto 0);
	SIGNAL QC, QW	:	STD_LOGIC_VECTOR(3 downto 0);
	SIGNAL QS		:	STD_LOGIC_VECTOR(3 downto 1);

 	SIGNAL Partial_carry		:	STD_LOGIC;
 	SIGNAL Partial_sum		:	STD_LOGIC;
 	SIGNAL Carry_in			:	STD_LOGIC;

BEGIN
-- Instantiation of components
Chip1	: Shift_Data_Reg_4bits             -- Reg A
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK	=> CLK,
		CD 	=> CD,
		S	=> LDA,
		X	=> A,
		Y	=> QA,
		RSI	=> '0',
		LSI => '0'
	);


Chip2	: Shift_Data_Reg_4bits               -- Reg B
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK	=> CLK,
		CD 	=> CD,
		S	=> LDA,
		X	=> B,
		Y	=> QB,
		RSI	=> '0',
		LSI => '0'
	);


Chip3	: Shift_Data_Reg_4bits                -- Reg Carry
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK		=> CLK,
		CD 		=> CD,
		S(1)	=> LDW,
		S(0)	=> '0',
		X		=> "0000",
		Y  		=> QC,
		RSI		=> '0',
		LSI 	=> Partial_carry
	);

Chip4	: Shift_Data_Reg_4bits           	-- Reg Accumulator (Working reg)
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK		=> CLK,
		CD 		=> CD,
		S(1)	=> '0',
		S(0)	=> LDW,
		X		=> "0000",
		Y  		=> QW,
		RSI		=> Partial_sum,
		LSI 	=> '0'
	);


Chip5	: Shift_Data_Reg_4bits           	-- Output register (Carry bit) 
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK				=> CLK,
		CD 				=> CD,
		S(1)				=> LDS,
		S(0)				=> LDS,
		X(3 downto 1)	=> "000",
		X(0)			=> QC(0),     			-- Partial_Carry_Out
		Y(3 downto 1)  	=> QS(3 downto 1),
		Y(0)			=> Cout, 
		RSI				=> '0',
		LSI 			=> '0'
	);
 

Chip6	: Shift_Data_Reg_4bits            -- Output register (Sum bits) 
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK				=> CLK,
		CD 				=> CD,
		S(1)				=> LDS,
		S(0)				=> LDS,
		X				=> QW,
		Y			  	=> S,
		RSI				=> '0',
		LSI 			=> '0'
	);


Chip7	: ONEBIT_ADDER          -- The arithmetic resource 
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		Ai		=> QA(0),
		Bi 	=> QB(0),
		Ci		=> Carry_in,
		So		=> Partial_sum,
		Co		=> Partial_carry
	);

Chip8	: mux4           
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		E		=> '1',
		Ch0 	=> Cin,
		Ch1		=> '0',
		Ch2		=> '0',
		Ch3		=> QC(0), 
		s(1)	=> Sel_Carry,
		s(0)	=> Sel_Carry,
		y		=> Carry_in

	);
	
Chip9	: Freq_Div_4           
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK		=> CLK,
		CD 		=> CD,
		CE		=> CE,
		TC4		=> TC4
	);
	
		
-- Extra logic in the circuit

END Structural;






